@@ -1,11 +1,12 @@
-# {{{ Version, package, new, etc
 
 package DBIx::SearchBuilder;
 
 use strict;
-use vars qw($VERSION);
 
-$VERSION = "1.26";
+use vars qw($VERSION);
+$VERSION = "1.38";
+
+use Clone qw();
 
 =head1 NAME
 
@@ -14,30 +15,74 @@ DBIx::SearchBuilder - Encapsulate SQL queries and rows in simple perl objects
 =head1 SYNOPSIS
 
   use DBIx::SearchBuilder;
+  
+  package My::Things;
+  use base qw/DBIx::SearchBuilder/;
+  
+  sub _Init {
+      my $self = shift;
+      $self->Table('Things');
+      return $self->SUPER::_Init(@_);
+  }
+  
+  sub NewItem {
+      my $self = shift;
+      # MyThing is a subclass of DBIx::SearchBuilder::Record
+      return(MyThing->new);
+  }
+  
+  package main;
 
   use DBIx::SearchBuilder::Handle;
   my $handle = DBIx::SearchBuilder::Handle->new();
   $handle->Connect( Driver => 'SQLite', Database => "my_test_db" );
 
-  my $sb = DBIx::SearchBuilder->new( Handle => $handle, Table => "my_table" );
+  my $sb = My::Things->new( Handle => $handle );
 
   $sb->Limit( FIELD => "column_1", VALUE => "matchstring" );
 
   while ( my $record = $sb->Next ) {
-    print $record->my_column_name();
+      print $record->my_column_name();
   }
 
 =head1 DESCRIPTION
 
 This module provides an object-oriented mechanism for retrieving and updating data in a DBI-accesible database. 
 
+In order to use this module, you should create a subclass of C<DBIx::SearchBuilder> and a 
+subclass of C<DBIx::SearchBuilder::Record> for each table that you wish to access.  (See
+the documentation of C<DBIx::SearchBuilder::Record> for more information on subclassing it.)
+
+Your C<DBIx::SearchBuilder> subclass must override C<NewItem>, and probably should override
+at least C<_Init> also; at the very least, C<_Init> should probably call C<_Handle> and C<_Table>
+to set the database handle (a C<DBIx::SearchBuilder::Handle> object) and table name for the class.
+You can try to override just about every other method here, as long as you think you know what you
+are doing.
+
+=head1 METHOD NAMING
+
+Each method has a lower case alias; '_' is used to separate words.
+For example, the method C<RedoSearch> has the alias C<redo_search>.
+
 =head1 METHODS
 
 =cut
 
-# {{{ sub new
 
-#instantiate a new object.
+=head2 new
+
+Creates a new SearchBuilder object and immediately calls C<_Init> with the same parameters
+that were passed to C<new>.  If you haven't overridden C<_Init> in your subclass, this means
+that you should pass in a C<DBIx::SearchBuilder::Handle> (or one of its subclasses) like this:
+
+   my $sb = My::DBIx::SearchBuilder::Subclass->new( Handle => $handle );
+
+However, if your subclass overrides _Init you do not need to take a Handle argument, as long
+as your subclass returns an appropriate handle object from the C<_Handle> method.  This is
+useful if you want all of your SearchBuilder objects to use a shared global handle and don't want
+to have to explicitly pass it in each time, for example.
+
+=cut
 
 sub new {
     my $proto = shift;
@@ -48,30 +93,33 @@ sub new {
     return ($self);
 }
 
-# }}}
 
-# {{{ sub _Init
 
-#Initialize the object
+=head2 _Init
+
+This method is called by C<new> with whatever arguments were passed to C<new>.  
+By default, it takes a C<DBIx::SearchBuilder::Handle> object as a C<Handle>
+argument, although this is not necessary if your subclass overrides C<_Handle>.
+
+=cut
 
 sub _Init {
     my $self = shift;
     my %args = ( Handle => undef,
                  @_ );
-    $self->{'DBIxHandle'} = $args{'Handle'};
+    $self->_Handle( $args{'Handle'} );
 
     $self->CleanSlate();
 }
 
-# }}}
 
-# {{{ sub CleanSlate
 
 =head2 CleanSlate
 
 This completely erases all the data in the SearchBuilder object. It's
-useful if a subclass is doing funky stuff to keep track of 
-a search
+useful if a subclass is doing funky stuff to keep track of a search and
+wants to reset the SearchBuilder data without losing its own data;
+it's probably cleaner to accomplish that in a different way, though.
 
 =cut
 
@@ -87,23 +135,68 @@ sub CleanSlate {
     $self->{'alias_count'}      = 0;
     $self->{'first_row'}        = 0;
     $self->{'must_redo_search'} = 1;
+    $self->{'show_rows'}        = 0;
     @{ $self->{'aliases'} } = ();
 
-    delete $self->{'items'}        if ( defined $self->{'items'} );
-    delete $self->{'left_joins'}   if ( defined $self->{'left_joins'} );
-    delete $self->{'raw_rows'}     if ( defined $self->{'raw_rows'} );
-    delete $self->{'count_all'}    if ( defined $self->{'count_all'} );
-    delete $self->{'subclauses'}   if ( defined $self->{'subclauses'} );
-    delete $self->{'restrictions'} if ( defined $self->{'restrictions'} );
+    delete $self->{$_} for qw(
+	items
+	left_joins
+	raw_rows
+	count_all
+	subclauses
+	restrictions
+	_open_parens
+	_close_parens
+    );
 
     #we have no limit statements. DoSearch won't work.
     $self->_isLimited(0);
-
 }
 
-# }}}
+=head2 Clone
+
+Returns copy of the current object with all search restrictions.
+
+=cut
+
+sub Clone
+{
+    my $self = shift;
+
+    my $obj = bless {}, ref($self);
+    %$obj = %$self;
+
+    delete $obj->{$_} for qw(
+        items
+    );
+    $obj->{'must_redo_search'} = 1;
+    $obj->{'itemscount'}       = 0;
+    
+    $obj->{$_} = Clone::clone($obj->{$_}) for ( $self->_ClonedAttributes );
+    return $obj;
+}
+
+=head2 _ClonedAttributes
+
+Returns list of the object's fields that should be copied.
+
+If your subclass store references in the object that should be copied while
+clonning then you probably want override this method and add own values to
+the list.
+
+=cut
+
+sub _ClonedAttributes
+{
+    return qw(
+        aliases
+        left_joins
+        subclauses
+        restrictions
+    );
+}
+
 
-# {{{ sub _Handle
 
 =head2 _Handle  [DBH]
 
@@ -119,9 +212,13 @@ sub _Handle {
     return ( $self->{'DBIxHandle'} );
 }
 
-# }}}
+=head2 _DoSearch
 
-# {{{ sub _DoSearch
+This internal private method actually executes the search on the database;
+it is called automatically the first time that you actually need results
+(such as a call to C<Next>).
+
+=cut
 
 sub _DoSearch {
     my $self = shift;
@@ -131,67 +228,83 @@ sub _DoSearch {
     # If we're about to redo the search, we need an empty set of items
     delete $self->{'items'};
 
-    eval {
-        
-        # TODO: finer-grained eval and cheking.
-       my  $records = $self->_Handle->SimpleQuery($QueryString);
-        my $counter;
-        $self->{'rows'} = 0;
-        while ( my $row = $records->fetchrow_hashref() ) {
-            my $item = $self->NewItem();
-            $item->LoadFromHash($row);
-            $self->AddRecord($item);
-        }
+    my $records = $self->_Handle->SimpleQuery($QueryString);
+    return 0 unless $records;
 
-        $self->{'must_redo_search'} = 0;
-    };
+    while ( my $row = $records->fetchrow_hashref() ) {
+	my $item = $self->NewItem();
+	$item->LoadFromHash($row);
+	$self->AddRecord($item);
+    }
+    return $self->_RecordCount if $records->err;
 
-    return ( $self->{'rows'});
+    $self->{'must_redo_search'} = 0;
+
+    return $self->_RecordCount;
 }
 
-# }}}
 
 =head2 AddRecord RECORD
 
-Adds a record object to this collection
+Adds a record object to this collection.
 
 =cut
 
 sub AddRecord {
     my $self = shift;
     my $record = shift;
-   push @{$self->{'items'}}, $record;
-   $self->{'rows'}++; 
+    push @{$self->{'items'}}, $record;
+}
+
+=head2 _RecordCount
+
+This private internal method returns the number of Record objects saved
+as a result of the last query.
+
+=cut
+
+sub _RecordCount {
+    my $self = shift;
+    return 0 unless defined $self->{'items'};
+    return scalar @{ $self->{'items'} };
 }
 
 
-# {{{ sub _DoCount
+
+=head2 _DoCount
+
+This internal private method actually executes a counting operation on the database;
+it is used by C<Count> and C<CountAll>.
+
+=cut
+
 
 sub _DoCount {
     my $self = shift;
     my $all  = shift || 0;
 
     my $QueryString = $self->BuildSelectCountQuery();
-    eval {
-        # TODO: finer-grained Eval
-        my $records     = $self->_Handle->SimpleQuery($QueryString);
+    my $records     = $self->_Handle->SimpleQuery($QueryString);
+    return 0 unless $records;
 
-        my @row = $records->fetchrow_array();
-        $self->{ $all ? 'count_all' : 'raw_rows' } = $row[0];
+    my @row = $records->fetchrow_array();
+    return 0 if $records->err;
 
-        return ( $row[0] );
-    };
+    $self->{ $all ? 'count_all' : 'raw_rows' } = $row[0];
+
+    return ( $row[0] );
 }
 
-# }}}
 
 
 =head2 _ApplyLimits STATEMENTREF
 
 This routine takes a reference to a scalar containing an SQL statement. 
-It massages the statement to limit the returned rows to $self->RowsPerPage
-starting with $self->FirstRow
-
+It massages the statement to limit the returned rows to only C<< $self->RowsPerPage >>
+rows, skipping C<< $self->FirstRow >> rows.  (That is, if rows are numbered
+starting from 0, row number C<< $self->FirstRow >> will be the first row returned.)
+Note that it probably makes no sense to set these variables unless you are also
+enforcing an ordering on the rows (with C<OrderByCols>, say).
 
 =cut
 
@@ -201,44 +314,28 @@ sub _ApplyLimits {
     my $statementref = shift;
     $self->_Handle->ApplyLimits($statementref, $self->RowsPerPage, $self->FirstRow);
     $$statementref =~ s/main\.\*/join(', ', @{$self->{columns}})/eg
-	if $self->{columns} and @{$self->{columns}};
-    if (my $groupby = $self->_GroupClause) {
-	$$statementref =~ s/(LIMIT \d+)?$/$groupby $1/;
-    }
-    
+	    if $self->{columns} and @{$self->{columns}};
 }
 
-# {{{ sub _DistinctQuery
 
 =head2 _DistinctQuery STATEMENTREF
 
 This routine takes a reference to a scalar containing an SQL statement. 
 It massages the statement to ensure a distinct result set is returned.
 
-
 =cut
 
 sub _DistinctQuery {
     my $self = shift;
     my $statementref = shift;
-    my $table = shift;
 
     # XXX - Postgres gets unhappy with distinct and OrderBy aliases
-    if (exists $self->{'order_clause'} && $self->{'order_clause'} =~ /(?<!main)\./) {
-        $$statementref = "SELECT main.* FROM $$statementref";
-    }
-    else {
-	$self->_Handle->DistinctQuery($statementref, $table)
-    }
+    $self->_Handle->DistinctQuery($statementref, $self)
 }
 
-# }}}
-
-# {{{ sub _BuildJoins
-
 =head2 _BuildJoins
 
-Build up all of the joins we need to perform this query
+Build up all of the joins we need to perform this query.
 
 =cut
 
@@ -250,12 +347,10 @@ sub _BuildJoins {
 
 }
 
-# }}}
-# {{{ sub _isJoined
 
 =head2 _isJoined 
 
-Returns true if this Searchbuilder requires joins between tables
+Returns true if this SearchBuilder will be joining multiple tables together.
 
 =cut
 
@@ -269,10 +364,8 @@ sub _isJoined {
 
 }
 
-# }}}
 
 
-# {{{ sub _LimitClause
 
 # LIMIT clauses are used for restricting ourselves to subsets of the search.
 
@@ -295,9 +388,7 @@ sub _LimitClause {
     return $limit_clause;
 }
 
-# }}}
 
-# {{{ sub _isLimited
 
 =head2 _isLimited
 
@@ -315,11 +406,8 @@ sub _isLimited {
     }
 }
 
-# }}}
 
-# }}} Private utility methods
 
-# {{{ BuildSelectQuery
 
 =head2 BuildSelectQuery
 
@@ -338,22 +426,20 @@ sub BuildSelectQuery {
 
     # DISTINCT query only required for multi-table selects
     if ($self->_isJoined) {
-        $self->_DistinctQuery(\$QueryString, $self->{'table'});
+        $self->_DistinctQuery(\$QueryString);
     } else {
         $QueryString = "SELECT main.* FROM $QueryString";
+        $QueryString .= $self->_GroupClause;
+        $QueryString .= $self->_OrderClause;
     }
 
-    $QueryString .= $self->_OrderClause;
-
     $self->_ApplyLimits(\$QueryString);
 
     return($QueryString)
 
 }
 
-# }}}
 
-# {{{ BuildSelectCountQuery
 
 =head2 BuildSelectCountQuery
 
@@ -383,11 +469,8 @@ sub BuildSelectCountQuery {
     return ($QueryString);
 }
 
-# }}}
 
-# {{{ Methods dealing traversing rows within the found set
 
-# {{{ sub Next
 
 =head2 Next
 
@@ -397,7 +480,7 @@ such that the following call to Next will start over with the first item retriev
 
 =cut
 
-*next = \&Next;
+
 
 sub Next {
     my $self = shift;
@@ -405,9 +488,9 @@ sub Next {
 
     return (undef) unless ( $self->_isLimited );
 
-    $self->_DoSearch() if ( $self->{'must_redo_search'} != 0 );
+    $self->_DoSearch() if $self->{'must_redo_search'};
 
-    if ( $self->{'itemscount'} < $self->{'rows'} ) {    #return the next item
+    if ( $self->{'itemscount'} < $self->_RecordCount ) {    #return the next item
         my $item = ( $self->{'items'}[ $self->{'itemscount'} ] );
         $self->{'itemscount'}++;
         return ($item);
@@ -418,9 +501,7 @@ sub Next {
     }
 }
 
-# }}}
 
-# {{{ sub GotoFirstItem
 
 =head2 GotoFirstItem
 
@@ -430,15 +511,13 @@ through the result set.
 
 =cut
 
-*goto_first_item = \&GotoFirstItem;
+
 sub GotoFirstItem {
     my $self = shift;
     $self->GotoItem(0);
 }
 
-# }}}
 
-# {{{ sub GotoItem
 
 
 =head2 GotoItem
@@ -455,9 +534,7 @@ sub GotoItem {
     $self->{'itemscount'} = $item;
 }
 
-# }}}
 
-# {{{ sub First
 
 =head2 First
 
@@ -471,9 +548,7 @@ sub First {
     return ( $self->Next );
 }
 
-# }}}
 
-# {{{ sub Last
 
 =head2 Last
 
@@ -487,9 +562,7 @@ sub Last {
     return ( $self->Next );
 }
 
-# }}}
 
-# {{{ ItemsArrayRef
 
 =head2 ItemsArrayRef
 
@@ -511,11 +584,8 @@ sub ItemsArrayRef {
     return ( $self->{'items'} || [] );
 }
 
-# }}}
 
-# }}}
 
-# {{{ sub NewItem
 
 =head2 NewItem
 
@@ -531,9 +601,7 @@ sub NewItem {
 "DBIx::SearchBuilder needs to be subclassed. you can't use it directly.\n";
 }
 
-# }}}
 
-# {{{ sub RedoSearch
 
 =head2 RedoSearch
 
@@ -547,11 +615,8 @@ sub RedoSearch {
     $self->{'must_redo_search'} = 1;
 }
 
-# }}}
 
-# {{{ Routines dealing with Restrictions (where subclauses)
 
-# {{{ sub UnLimit
 
 =head2 UnLimit
 
@@ -565,9 +630,7 @@ sub UnLimit {
     $self->_isLimited(-1);
 }
 
-# }}}
 
-# {{{ sub Limit
 
 =head2 Limit
 
@@ -618,6 +681,11 @@ STARTSWITH is like LIKE, except it only appends a % at the end of the string
 
 ENDSWITH is like LIKE, except it prepends a % to the beginning of the string
 
+=item "MATCHES"
+
+MATCHES is equivalent to the database's LIKE -- that is, it's actually LIKE, but
+doesn't surround the string in % signs as LIKE does.
+
 =back
 
 =item ENTRYAGGREGATOR 
@@ -636,7 +704,7 @@ this search case sensitive
 sub Limit {
     my $self = shift;
     my %args = (
-        TABLE           => $self->{'table'},
+        TABLE           => $self->Table,
         FIELD           => undef,
         VALUE           => undef,
         ALIAS           => undef,
@@ -667,7 +735,9 @@ sub Limit {
         elsif ( $args{'OPERATOR'} =~ /ENDSWITH/i ) {
             $args{'VALUE'}    = "%" . $args{'VALUE'};
             $args{'OPERATOR'} = "LIKE";
-        }
+        } 
+	
+	$args{'OPERATOR'} =~ s/MATCHES/LIKE/i;  # MATCHES becomes LIKE, with no % stuff
 
         #if we're explicitly told not to to quote the value or
         # we're doing an IS or IS NOT (null), don't quote the operator.
@@ -703,9 +773,7 @@ sub Limit {
     }
 }
 
-# }}}
 
-# {{{ sub ShowRestrictions
 
 =head2 ShowRestrictions
 
@@ -723,9 +791,7 @@ sub ShowRestrictions {
 
 }
 
-# }}}
 
-# {{{ sub ImportRestrictions
 
 =head2 ImportRestrictions
 
@@ -741,13 +807,11 @@ sub ImportRestrictions {
     $self->{'where_clause'} = shift;
 }
 
-# }}}
 
-# {{{ sub _GenericRestriction
 
 sub _GenericRestriction {
     my $self = shift;
-    my %args = ( TABLE           => $self->{'table'},
+    my %args = ( TABLE           => $self->Table,
                  FIELD           => undef,
                  VALUE           => undef,
                  ALIAS           => undef,
@@ -777,7 +841,7 @@ sub _GenericRestriction {
     unless ( $args{'ALIAS'} ) {
 
         #if the table we're looking at is the same as the main table
-        if ( $args{'TABLE'} eq $self->{'table'} ) {
+        if ( $args{'TABLE'} eq $self->Table ) {
 
             # TODO this code assumes no self joins on that table.
             # if someone can name a case where we'd want to do that,
@@ -862,9 +926,7 @@ sub _GenericRestriction {
 
 }
 
-# }}}
 
-# {{{ Parentheses Control
 sub _OpenParen {
     my ( $self, $clause ) = @_;
     $self->{_open_parens}{$clause}++;
@@ -882,9 +944,7 @@ sub _CloseParen {
     }
 }
 
-# }}}
 
-# {{{ sub _AddRestriction
 sub _AddSubClause {
     my $self      = shift;
     my $clauseid  = shift;
@@ -894,9 +954,7 @@ sub _AddSubClause {
 
 }
 
-# }}}
 
-# {{{ sub _WhereClause
 
 sub _WhereClause {
     my $self = shift;
@@ -926,9 +984,7 @@ sub _WhereClause {
 
 }
 
-# }}}
 
-# {{{ sub _CompileGenericRestrictions
 
 #Compile the restrictions to a WHERE Clause
 
@@ -948,31 +1004,26 @@ sub _CompileGenericRestrictions {
     }
 }
 
-# }}}
 
-# }}}
 
-# {{{ Routines dealing with ordering
 
-# {{{ sub OrderBy
 
-=head2 Orderby PARAMHASH
+=head2 OrderBy PARAMHASH
 
-Orders the returned results by ALIAS.FIELD ORDER. (by default 'main.id ASC')
+Orders the returned results by ALIAS.FIELD ORDER.
 
 Takes a paramhash of ALIAS, FIELD and ORDER.  
-ALIAS defaults to main
-FIELD defaults to the primary key of the main table.  Also accepts C<FUNCTION(FIELD)> format
-ORDER defaults to ASC(ending).  DESC(ending) is also a valid value for OrderBy
+ALIAS defaults to C<main>.
+FIELD has no default value.
+ORDER defaults to ASC(ending). DESC(ending) is also a valid value for OrderBy.
 
+FIELD also accepts C<FUNCTION(FIELD)> format.
 
 =cut
 
 sub OrderBy {
     my $self = shift;
-    my %args = ( @_ );
-
-    $self->OrderByCols( \%args );
+    $self->OrderByCols( { @_ } );
 }
 
 =head2 OrderByCols ARRAY
@@ -985,10 +1036,24 @@ The result set is ordered by the items in the array.
 sub OrderByCols {
     my $self = shift;
     my @args = @_;
-    my $row;
-    my $clause;
 
-    foreach $row ( @args ) {
+    $self->{'order_by'} = \@args;
+    $self->RedoSearch();
+}
+
+=head2 _OrderClause
+
+returns the ORDER BY clause for the search.
+
+=cut
+
+sub _OrderClause {
+    my $self = shift;
+
+    return '' unless $self->{'order_by'};
+
+    my $clause = '';
+    foreach my $row ( @{$self->{'order_by'}} ) {
 
         my %rowhash = ( ALIAS => 'main',
 			FIELD => undef,
@@ -1018,42 +1083,14 @@ sub OrderByCols {
             $clause .= $rowhash{'ORDER'};
         }
     }
+    $clause = " ORDER BY$clause " if $clause;
 
-    if ($clause) {
-	$self->{'order_clause'} = "ORDER BY" . $clause;
-    }
-    else {
-	$self->{'order_clause'} = "";
-    }
-    $self->RedoSearch();
+    return $clause;
 }
 
-# }}} 
 
-# {{{ sub _OrderClause
 
-=head2 _OrderClause
 
-returns the ORDER BY clause for the search.
-
-=cut
-
-sub _OrderClause {
-    my $self = shift;
-
-    unless ( defined $self->{'order_clause'} ) {
-	return "";
-    }
-    return ($self->{'order_clause'});
-}
-
-# }}}
-
-# }}}
-
-# {{{ Routines dealing with grouping
-
-# {{{ GroupBy (OBSOLETE)
 
 =head2 GroupBy  (DEPRECATED)
 
@@ -1061,11 +1098,9 @@ Alias for the GroupByCols method.
 
 =cut
 
-sub GroupBy { (shift)->GroupByCols( @_) }
+sub GroupBy { (shift)->GroupByCols( @_ ) }
 
-# }}}
 
-# {{{ GroupByCols
 
 =head2 GroupByCols ARRAY_OF_HASHES
 
@@ -1076,35 +1111,11 @@ Each hash contains the keys ALIAS and FIELD. ALIAS defaults to 'main' if ignored
 sub GroupByCols {
     my $self = shift;
     my @args = @_;
-    my $row;
-    my $clause;
 
-    foreach $row ( @args ) {
-        my %rowhash = ( ALIAS => 'main',
-			FIELD => undef,
-			%$row
-		      );
-
-        if ( ($rowhash{'ALIAS'}) and
-             ($rowhash{'FIELD'}) ) {
-
-            $clause .= ($clause ? ", " : " ");
-            $clause .= $rowhash{'ALIAS'} . ".";
-            $clause .= $rowhash{'FIELD'};
-        }
-    }
-
-    if ($clause) {
-	$self->{'group_clause'} = "GROUP BY" . $clause;
-    }
-    else {
-	$self->{'group_clause'} = "";
-    }
+    $self->{'group_by'} = \@args;
     $self->RedoSearch();
 }
-# }}} 
 
-# {{{ _GroupClause
 
 =head2 _GroupClause
 
@@ -1114,20 +1125,41 @@ Private function to return the "GROUP BY" clause for this query.
 
 sub _GroupClause {
     my $self = shift;
+    return '' unless $self->{'group_by'};
 
-    unless ( defined $self->{'group_clause'} ) {
-	    return "";
+    my $row;
+    my $clause;
+
+    foreach $row ( @{$self->{'group_by'}} ) {
+        my %rowhash = ( ALIAS => 'main',
+			FIELD => undef,
+			%$row
+		      );
+        if ($rowhash{'FUNCTION'} ) {
+            $clause .= ($clause ? ", " : " ");
+            $clause .= $rowhash{'FUNCTION'};
+
+        }
+        elsif ( ($rowhash{'ALIAS'}) and
+             ($rowhash{'FIELD'}) ) {
+
+            $clause .= ($clause ? ", " : " ");
+            $clause .= $rowhash{'ALIAS'} . ".";
+            $clause .= $rowhash{'FIELD'};
+        }
+    }
+
+    if ($clause) {
+	return " GROUP BY" . $clause . " ";
+    }
+    else {
+	return '';
     }
-    return ($self->{'group_clause'});
 }
 
-# }}}
 
-# }}}
 
-# {{{ routines dealing with table aliases and linking tables
 
-# {{{ sub NewAlias
 
 =head2 NewAlias
 
@@ -1150,9 +1182,7 @@ sub NewAlias {
     return $alias;
 }
 
-# }}}
 
-# {{{ sub _GetAlias
 
 # _GetAlias is a private function which takes an tablename and
 # returns a new alias for that table without adding something
@@ -1170,9 +1200,7 @@ sub _GetAlias {
 
 }
 
-# }}}
 
-# {{{ sub Join
 
 =head2 Join
 
@@ -1210,34 +1238,24 @@ sub Join {
 
 }
 
-# }}}
 
-# }}}
 
-# {{{ Deal with 'pages' of results'
 
-# {{{ sub NextPage
 
 sub NextPage {
     my $self = shift;
     $self->FirstRow( $self->FirstRow + $self->RowsPerPage );
 }
 
-# }}}
 
-# {{{ sub FirstPage
 sub FirstPage {
     my $self = shift;
     $self->FirstRow(1);
 }
 
-# }}}
 
-# {{{ sub LastPage
 
-# }}}
 
-# {{{ sub PrevPage
 
 sub PrevPage {
     my $self = shift;
@@ -1249,9 +1267,7 @@ sub PrevPage {
     }
 }
 
-# }}}
 
-# {{{ sub GotoPage
 
 sub GotoPage {
     my $self = shift;
@@ -1264,9 +1280,7 @@ sub GotoPage {
     }
 }
 
-# }}}
 
-# {{{ sub RowsPerPage
 
 =head2 RowsPerPage
 
@@ -1283,9 +1297,7 @@ sub RowsPerPage {
     return ( $self->{'show_rows'} );
 }
 
-# }}}
 
-# {{{ sub FirstRow
 
 =head2 FirstRow
 
@@ -1311,13 +1323,9 @@ sub FirstRow {
     return ( $self->{'first_row'} );
 }
 
-# }}}
 
-# }}}
 
-# {{{ Public utility methods
 
-# {{{ sub _ItemsCounter
 
 =head2 _ItemsCounter
 
@@ -1330,9 +1338,7 @@ sub _ItemsCounter {
     return $self->{'itemscount'};
 }
 
-# }}}
 
-# {{{ sub Count
 
 =head2 Count
 
@@ -1341,7 +1347,7 @@ Returns the number of records in the set.
 =cut
 
 
-*count = \&Count;
+
 sub Count {
     my $self = shift;
 
@@ -1363,13 +1369,11 @@ sub Count {
     # If we have loaded everything from the DB we have an
     # accurate count already.
     else {
-        return ( $self->{'rows'} );
+        return $self->_RecordCount;
     }
 }
 
-# }}}
 
-# {{{ sub CountAll
 
 =head2 CountAll
 
@@ -1418,14 +1422,12 @@ sub CountAll {
     # If we have loaded everything from the DB we have an
     # accurate count already.
     else {
-        return ( $self->{'rows'} );
+        return $self->_RecordCount;
     }
 }
 
-# }}}
 
 
-# {{{ sub IsLast
 
 =head2 IsLast
 
@@ -1436,17 +1438,17 @@ Returns true if the current row is the last record in the set.
 sub IsLast {
     my $self = shift;
 
+    return undef unless $self->Count;
+
     if ( $self->_ItemsCounter == $self->Count ) {
         return (1);
     }
     else {
-        return (undef);
+        return (0);
     }
 }
 
-# }}}
 
-# {{{ sub DEBUG
 
 sub DEBUG {
     my $self = shift;
@@ -1456,14 +1458,11 @@ sub DEBUG {
     return ( $self->{'DEBUG'} );
 }
 
-# }}}
 
 
 
 
-# }}}
 
-# {{{ Column
 
 =head2 Column { FIELD => undef } 
 
@@ -1489,7 +1488,7 @@ sub Column {
             $alias;
         }
         else {
-            $self->{table};
+            $self->Table;
         }
     };
 
@@ -1498,20 +1497,26 @@ sub Column {
         if ( $func =~ /^DISTINCT\s*COUNT$/i ) {
             $name = "COUNT(DISTINCT $name)";
         }
-        else {
-            $name = "\U$func\E($name)";
+        # If we want to substitute 
+        elsif ($func =~ /\?/) {
+            $name = join($name,split(/\?/,$func));
         }
+        # If we want to call a simple function on the column
+        elsif ($func !~ /\(/)  {
+            $name = "\U$func\E($name)";
+        } else {
+            $name = $func;
+        }
+        
     }
 
     my $column = "col" . @{ $self->{columns} ||= [] };
-    $column = $args{FIELD} if $table eq $self->{table} and !$args{ALIAS};
+    $column = $args{FIELD} if $table eq $self->Table and !$args{ALIAS};
     push @{ $self->{columns} }, "$name AS \L$column";
     return $column;
 }
 
-# }}}
 
-# {{{ Columns 
 
 
 =head2 Columns LIST
@@ -1525,12 +1530,10 @@ sub Columns {
     $self->Column( FIELD => $_ ) for @_;
 }
 
-# }}}
 
-# {{{ Fields
 
 =head2 Fields TABLE
- 
+
 Return a list of fields in TABLE, lowercased.
 
 TODO: Why are they lowercased?
@@ -1555,10 +1558,8 @@ sub Fields {
       };
 }
 
-# }}}
 
 
-# {{{ HasField
 
 =head2 HasField  { TABLE => undef, FIELD => undef }
 
@@ -1578,13 +1579,11 @@ sub HasField {
     return grep { $_ eq $field } $self->Fields($table);
 }
 
-# }}}
 
-# {{{ SetTable
 
 =head2 Table [TABLE]
 
-If called with an arguemnt, sets this collection's table.
+If called with an argument, sets this collection's table.
 
 Always returns this collection's table.
 
@@ -1602,19 +1601,33 @@ sub Table {
 }
 
 
-# }}}
-
+if( eval { require capitalization } ) {
+	capitalization->unimport( __PACKAGE__ );
+}
 
 1;
 __END__
 
-# {{{ POD
 
 
+=head1 TESTING
+
+In order to test most of the features of C<DBIx::SearchBuilder>, you need
+to provide C<make test> with a test database.  For each DBI driver that you
+would like to test, set the environment variables C<SB_TEST_FOO>, C<SB_TEST_FOO_USER>,
+and C<SB_TEST_FOO_PASS> to a database name, database username, and database password,
+where "FOO" is the driver name in all uppercase.  You can test as many drivers
+as you like.  (The appropriate C<DBD::> module needs to be installed in order for
+the test to work.)  Note that the C<SQLite> driver will automatically be tested if C<DBD::Sqlite>
+is installed, using a temporary file as the database.  For example:
+
+  SB_TEST_MYSQL=test SB_TEST_MYSQL_USER=root SB_TEST_MYSQL_PASS=foo \
+    SB_TEST_PG=test SB_TEST_PG_USER=postgres  make test
+
 
 =head1 AUTHOR
 
-Copyright (c) 2001-2004 Jesse Vincent, jesse@fsck.com.
+Copyright (c) 2001-2005 Jesse Vincent, jesse@fsck.com.
 
 All rights reserved.
 
@@ -1628,7 +1641,6 @@ DBIx::SearchBuilder::Handle, DBIx::SearchBuilder::Record.
 
 =cut
 
-# }}}
 
 
 
